//
//  GeneticSearchGlobal.h
//  AroniSmartLytics
//
//  Created by felicien kanyamibwa on 4/30/14.
//  Copyright (c) 2014 AroniSoft LLC. All rights reserved.
//

/**
 <!-- globalinfo-start -->
 * This Bayes Network learning algorithm uses genetic search for finding a well scoring Bayes network structure. Genetic search works by having a population of Bayes network structures and allow them to mutate and apply cross over to get offspring. The best network structure found during the process is returned.
 * <p/>
 <!-- globalinfo-end -->
 *
 <!-- options-start -->
 * Valid options are: <p/>
 *
 * <pre> -L &lt;integer&gt;
 *  Population size</pre>
 *
 * <pre> -A &lt;integer&gt;
 *  Descendant population size</pre>
 *
 * <pre> -U &lt;integer&gt;
 *  Number of runs</pre>
 *
 * <pre> -M
 *  Use mutation.
 *  (default true)</pre>
 *
 * <pre> -C
 *  Use cross-over.
 *  (default true)</pre>
 *
 * <pre> -O
 *  Use tournament selection (true) or maximum subpopulatin (false).
 *  (default false)</pre>
 *
 * <pre> -R &lt;seed&gt;
 *  Random number seed</pre>
 *
 * <pre> -mbc
 *  Applies a Markov Blanket correction to the network structure,
 *  after a network structure is learned. This ensures that all
 *  nodes in the network are part of the Markov blanket of the
 *  classifier node.</pre>
 *
 * <pre> -S [LOO-CV|k-Fold-CV|Cumulative-CV]
 *  Score type (LOO-CV,k-Fold-CV,Cumulative-CV)</pre>
 *
 * <pre> -Q
 *  Use probabilistic or 0/1 scoring.
 *  (default probabilistic scoring)</pre>
 *
 <!-- options-end -->
 */


#import "GlobalScoreSearchAlgorithm.h"


@interface GeneticSearchGlobal : GlobalScoreSearchAlgorithm
{
    /** number of runs **/
    NSInteger m_nRuns;
    
	/** size of population **/
	NSInteger m_nPopulationSize;
    
	/** size of descendant population **/
	NSInteger m_nDescendantPopulationSize;
    
	/** use cross-over? **/
	BOOL m_bUseCrossOver;
    
	/** use mutation? **/
	BOOL m_bUseMutation;
	
	/** use tournament selection or take best sub-population **/
	BOOL m_bUseTournamentSelection;
	
	/** random number seed **/
	NSInteger m_nSeed;
    
	/** used in BayesNetRepresentation for efficiently determining
	 * whether a number is square
	 */
	NSMutableArray* g_bIsSquare;
    
    
}
/**
* search determines the network structure/graph of the network
* with a genetic search algorithm.
*
* @param bayesNet the network to use
* @param instances the data to use
* @throws Exception if population size doesn fit or neither cross-over or mutation was chosen
*/
-(void)search: (BayesNet*) bayesNet instances:(InstancesDataset*) instances;

/** copyParentSets copies parent sets of source to dest BayesNet
 * @param dest destination network
 * @param source source network
 */
-(void) copyParentSets:(BayesNet *) dest source: (BayesNet*) source;
/**
 * @return number of runs
 */
-(NSInteger) getRuns ;
/**
 * Sets the number of runs
 * @param nRuns The number of runs to set
 */
-(void) setRuns: (NSInteger) nRuns;
/**
 * Returns an enumeration describing the available options.
 *
 * @return an enumeration of all the available options.
 */
-(NSEnumerator*) listOptions;
/**
 * Parses a given list of options. <p/>
 *
 <!-- options-start -->
 * Valid options are: <p/>
 *
 * <pre> -L &lt;integer&gt;
 *  Population size</pre>
 *
 * <pre> -A &lt;integer&gt;
 *  Descendant population size</pre>
 *
 * <pre> -U &lt;integer&gt;
 *  Number of runs</pre>
 *
 * <pre> -M
 *  Use mutation.
 *  (default true)</pre>
 *
 * <pre> -C
 *  Use cross-over.
 *  (default true)</pre>
 *
 * <pre> -O
 *  Use tournament selection (true) or maximum subpopulatin (false).
 *  (default false)</pre>
 *
 * <pre> -R &lt;seed&gt;
 *  Random number seed</pre>
 *
 * <pre> -mbc
 *  Applies a Markov Blanket correction to the network structure,
 *  after a network structure is learned. This ensures that all
 *  nodes in the network are part of the Markov blanket of the
 *  classifier node.</pre>
 *
 * <pre> -S [LOO-CV|k-Fold-CV|Cumulative-CV]
 *  Score type (LOO-CV,k-Fold-CV,Cumulative-CV)</pre>
 *
 * <pre> -Q
 *  Use probabilistic or 0/1 scoring.
 *  (default probabilistic scoring)</pre>
 *
 <!-- options-end -->
 *
 * @param options the list of options as an array of strings
 * @throws Exception if an option is not supported
 */
-(void) setOptions: (NSMutableArray*) options;
/**
 * Gets the current settings of the search algorithm.
 *
 * @return an array of strings suitable for passing to setOptions
 */
-(NSMutableArray*) getOptions;
/**
 * @return whether cross-over is used
 */
-(BOOL) getUseCrossOver;

/**
 * @return whether mutation is used
 */
-(BOOL) getUseMutation;
/**
 * @return descendant population size
 */
-(NSInteger)  getDescendantPopulationSize;

/**
 * @return population size
 */
-(NSInteger)  getPopulationSize;

/**
 * @param bUseCrossOver sets whether cross-over is used
 */
-(void) setUseCrossOver: (BOOL) bUseCrossOver ;

/**
 * @param bUseMutation sets whether mutation is used
 */
-(void) setUseMutation: (BOOL) bUseMutation;
/**
 * @return whether Tournament Selection (true) or Maximum Sub-Population (false) should be used
 */
-(BOOL) getUseTournamentSelection ;

/**
 * @param bUseTournamentSelection sets whether Tournament Selection or Maximum Sub-Population should be used
 */
-(void) setUseTournamentSelection: (BOOL) bUseTournamentSelection;

/**
 * @param iDescendantPopulationSize sets descendant population size
 */
-(void) setDescendantPopulationSize: (NSInteger) iDescendantPopulationSize;
/**
 * @param iPopulationSize sets population size
 */
-(void) setPopulationSize: (NSInteger) iPopulationSize ;
/**
 * @return random number seed
 */
-(NSInteger)  getSeed;
/**
 * Sets the random number seed
 * @param nSeed The number of the seed to set
 */
-(void) setSeed:(NSInteger) nSeed ;
/**
 * This will return a string describing the classifier.
 * @return The string.
 */
-(NSString*)  globalInfo;
/**
 * @return a string to describe the Runs option.
 */
-(NSString*)  runsTipText;
/**
 * @return a string to describe the Seed option.
 */
-(NSString*)  seedTipText;
/**
 * @return a string to describe the Population Size option.
 */
-(NSString*)  populationSizeTipText;
// populationSizeTipText

/**
 * @return a string to describe the Descendant Population Size option.
 */
-(NSString*)  descendantPopulationSizeTipText;
/**
 * @return a string to describe the Use Mutation option.
 */
-(NSString*)  useMutationTipText;
// useMutationTipText

/**
 * @return a string to describe the Use Cross-Over option.
 */
-(NSString*)  useCrossOverTipText;
/**
 * @return a string to describe the Use Tournament Selection option.
 */
-(NSString*)  useTournamentSelectionTipText;
@end
